/*==============================================================================*/
/*  Copyright (C) 2020, Heartland Data inc. All Rights Reserved.                */
/*                                                                              */
/*  Title  :   Android Framework Data Writing Driver                            */
/*  EventID:   Via the driver                                                   */
/*  AppVer :   DT+ Ver1.00`                                                   */
/*  FileID :   43b                                                              */
/*  Version:   1.0                                                              */
/*  Author :   HLDC                                                             */
/*==============================================================================*/
/*==============================================================================*/
/*  Desc:   Header for File IO                                                  */
/*==============================================================================*/
#include <stdio.h>
#include <sys/types.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>

/*==============================================================================*/
/*  Macro:  DT_UINT                                                             */
/*  Desc:   Please change Test Point argument type for DT+ Project setting.     */
/*==============================================================================*/
#define	DT_UINT unsigned int

/*==============================================================================*/
/*  Macro:  DT_DRV                                                              */
/*  Desc:   Register Proc file name for Linux.                                  */
/*==============================================================================*/
#define 	DT_DRV "/proc/dtproc"

/*==============================================================================*/
/*  Struct: DT_DATA                                                             */
/*  Desc:   Test Point data.                                                    */
/*==============================================================================*/
struct DT_DATA {
	char tp_func;
	DT_UINT addr;
	DT_UINT dat;
	DT_UINT size;
	unsigned char value[256];
};

/*==============================================================================*/
/*  Desc:   DT_DATA.tp_func ID                                                  */
/*==============================================================================*/
#define TP_BusOut			'3'
#define TP_MemoryOutput		'4'

static int fd = -1;

/*==============================================================================*/
/*  Func:   _TP_BusOut                                                          */
/*  Desc:   Called by Test Point                                                */
/*==============================================================================*/
void _TP_BusOut( DT_UINT addr, DT_UINT data )
{
	struct DT_DATA dt_data;

	if (fd < 0) {
		fd = open(DT_DRV, O_RDWR);
		if (fd <= 0) {
			return;
		}
	}

	dt_data.tp_func = TP_BusOut;
	dt_data.addr = addr;
	dt_data.dat = data;

	write(fd, &dt_data, sizeof(dt_data));
}

/*==============================================================================*/
/*  Func:   _TP_MemoryOutput                                                    */
/*  Desc:   Called by Variable TestPoint                                        */
/*==============================================================================*/
void _TP_MemoryOutput( DT_UINT addr, DT_UINT data, void *value, DT_UINT size)
{
	struct DT_DATA dt_data;

	if (fd < 0) {
		fd = open(DT_DRV, O_RDWR);
		if (fd <= 0) {
			return;
		}
	}

	if (size > 256) size = 256;
	dt_data.tp_func = TP_MemoryOutput;
	dt_data.addr = addr;
	dt_data.dat = data;
	dt_data.size = size;
	memcpy(dt_data.value, value, size);

	write(fd, &dt_data, sizeof(dt_data));
}
